@@ -4,7 +4,7 @@
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/QueryExecutorImpl.java,v 1.22 2009/07/01 05:00:40 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/QueryExecutorImpl.java,v 1.22.2.1 2009/12/04 19:53:27 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -607,7 +607,7 @@ public class QueryExecutorImpl implements QueryExecutor {
     private final PGStream pgStream;
     private final Logger logger;
 
-    public CopyOperation startCopy(String sql) throws SQLException {
+    public CopyOperation startCopy(String sql, boolean suppressBegin) throws SQLException {
         throw new PSQLException(GT.tr("Copy not implemented for protocol version 2"), PSQLState.NOT_IMPLEMENTED);
     }
 }
