@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3g/Jdbc3gArray.java,v 1.4 2005/01/11 08:25:47 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3g/Jdbc3gArray.java,v 1.6 2008/01/08 06:56:30 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -16,9 +16,9 @@ import java.sql.ResultSet;
 
 public class Jdbc3gArray extends org.postgresql.jdbc2.AbstractJdbc2Array implements java.sql.Array
 {
-    public Jdbc3gArray(BaseConnection conn, int idx, Field field, BaseResultSet rs) throws SQLException
+    public Jdbc3gArray(BaseConnection conn, int oid, String fieldString) throws SQLException
     {
-        super(conn, idx, field, rs);
+        super(conn, oid, fieldString);
     }
 
     public Object getArray(Map < String, Class < ? >> map) throws SQLException
