@@ -3,7 +3,7 @@
 * Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.44 2008/11/07 09:11:37 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.44.2.2 2009/12/09 01:06:41 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -326,6 +326,41 @@ public class DatabaseMetaDataTest extends TestCase
         rs.close();
     }
 
+    public void testDroppedColumns() throws SQLException
+    {
+        if (!TestUtil.haveMinimumServerVersion(con, "8.4"))
+            return;
+
+        Statement stmt = con.createStatement();
+        stmt.execute("ALTER TABLE testmetadata DROP name");
+        stmt.execute("ALTER TABLE testmetadata DROP colour");
+        stmt.close();
+
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getColumns(null, null, "testmetadata", null);
+
+        assertTrue(rs.next());
+        assertEquals("id", rs.getString("COLUMN_NAME"));
+        assertEquals(1, rs.getInt("ORDINAL_POSITION"));
+
+        assertTrue(rs.next());
+        assertEquals("updated", rs.getString("COLUMN_NAME"));
+        assertEquals(2, rs.getInt("ORDINAL_POSITION"));
+
+        assertTrue(rs.next());
+        assertEquals("quest", rs.getString("COLUMN_NAME"));
+        assertEquals(3, rs.getInt("ORDINAL_POSITION"));
+
+        rs.close();
+
+        rs = dbmd.getColumns(null, null, "testmetadata", "quest");
+        assertTrue(rs.next());
+        assertEquals("quest", rs.getString("COLUMN_NAME"));
+        assertEquals(3, rs.getInt("ORDINAL_POSITION"));
+        assertTrue(!rs.next());
+        rs.close();
+    }
+
     public void testSerialColumns() throws SQLException
     {
         DatabaseMetaData dbmd = con.getMetaData();
@@ -447,6 +482,25 @@ public class DatabaseMetaDataTest extends TestCase
         rs.close();
     }
 
+    public void testPartialIndexInfo() throws SQLException
+    {
+        Statement stmt = con.createStatement();
+        stmt.execute("create index idx_p_name_id on testmetadata (name) where id > 5");
+        stmt.close();
+
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getIndexInfo(null, null, "testmetadata", false, false);
+
+        assertTrue(rs.next());
+        assertEquals("idx_p_name_id", rs.getString("INDEX_NAME"));
+        assertEquals(1, rs.getInt("ORDINAL_POSITION"));
+        assertEquals("name", rs.getString("COLUMN_NAME"));
+        assertEquals("(id > 5)", rs.getString("FILTER_CONDITION"));
+        assertTrue(rs.getBoolean("NON_UNIQUE"));
+
+        rs.close();
+    }
+
     public void testTableTypes() throws SQLException
     {
         // At the moment just test that no exceptions are thrown KJ
