@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/PreparedStatementTest.java,v 1.18 2006/12/01 08:53:46 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/PreparedStatementTest.java,v 1.23 2009/04/20 21:44:09 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -39,12 +39,14 @@ public class PreparedStatementTest extends TestCase
         conn = TestUtil.openDB();
         TestUtil.createTable(conn, "streamtable", "bin bytea, str text");
         TestUtil.createTable(conn, "texttable", "ch char(3), te text, vc varchar(3)");
+        TestUtil.createTable(conn, "intervaltable", "i interval");
     }
 
     protected void tearDown() throws SQLException
     {
         TestUtil.dropTable(conn, "streamtable");
         TestUtil.dropTable(conn, "texttable");
+        TestUtil.dropTable(conn, "intervaltable");
         TestUtil.closeDB(conn);
     }
 
@@ -405,6 +407,27 @@ public class PreparedStatementTest extends TestCase
         st.close();
     }
     
+    public void testDollarQuotesAndIdentifiers() throws SQLException {
+        // dollar-quotes are supported in the backend since version 8.0
+        if (!TestUtil.haveMinimumServerVersion(conn, "8.0"))
+            return;
+        
+        PreparedStatement st;
+        
+        conn.createStatement().execute("CREATE TEMP TABLE a$b$c(a varchar, b varchar)");
+        st = conn.prepareStatement("INSERT INTO a$b$c (a, b) VALUES (?, ?)");
+        st.setString(1, "a");
+        st.setString(2, "b");
+        st.executeUpdate();
+        st.close();
+
+        conn.createStatement().execute("CREATE TEMP TABLE e$f$g(h varchar, e$f$g varchar) ");
+        st = conn.prepareStatement("UPDATE e$f$g SET h = ? || e$f$g");
+        st.setString(1, "a");
+        st.executeUpdate();
+        st.close();
+    }
+    
     public void testComments() throws SQLException {
         PreparedStatement st;
         ResultSet rs;
@@ -758,4 +781,43 @@ public class PreparedStatementTest extends TestCase
         pstmt.close();
         
     }
+
+    public void testUnknownSetObject() throws SQLException
+    {
+        PreparedStatement pstmt = conn.prepareStatement("INSERT INTO intervaltable(i) VALUES (?)");
+
+        if (TestUtil.isProtocolVersion(conn, 3))
+        {
+            pstmt.setString(1, "1 week");
+            try {
+                pstmt.executeUpdate();
+                fail("Should have failed with type mismatch.");
+            } catch (SQLException sqle) {
+            }
+        }
+
+        pstmt.setObject(1, "1 week", Types.OTHER);
+        pstmt.executeUpdate();
+        pstmt.close();
+    }
+
+    /**
+     * When we have parameters of unknown type and it's not using
+     * the unnamed statement, we issue a protocol level statment
+     * describe message for the V3 protocol.  This test just makes
+     * sure that works.
+     */
+    public void testStatementDescribe() throws SQLException
+    {
+        PreparedStatement pstmt = conn.prepareStatement("SELECT ?::int");
+        pstmt.setObject(1, new Integer(2), Types.OTHER);
+        for (int i=0; i<10; i++) {
+            ResultSet rs = pstmt.executeQuery();
+            assertTrue(rs.next());
+            assertEquals(2, rs.getInt(1));
+            rs.close();
+        }
+        pstmt.close();
+    }
+
 }
