@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2005, PostgreSQL Global Development Group
+* Copyright (c) 2005-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/Jdbc3BlobTest.java,v 1.3 2005/11/24 02:31:44 oliver Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/Jdbc3BlobTest.java,v 1.6 2008/01/08 06:56:31 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -14,7 +14,6 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.sql.Blob;
 import java.sql.Connection;
-import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
@@ -112,6 +111,41 @@ public class Jdbc3BlobTest extends TestCase
         readWrite(data);
     }
 
+    public void testTruncate() throws SQLException
+    {
+        if (!TestUtil.haveMinimumServerVersion(_conn, "8.3"))
+            return;
+
+        byte data[] = new byte[100];
+        for (byte i=0; i<data.length; i++) {
+            data[i] = i;
+        }
+        readWrite(data);
+
+        PreparedStatement ps = _conn.prepareStatement(SELECT);
+        ResultSet rs = ps.executeQuery();
+        
+        assertTrue(rs.next());
+        Blob blob = rs.getBlob("DATA");
+
+        assertEquals(100, blob.length());
+
+        blob.truncate(50);
+        assertEquals(50, blob.length());
+
+        blob.truncate(150);
+        assertEquals(150, blob.length());
+
+        data = blob.getBytes(1, 200);
+        assertEquals(150, data.length);
+        for (byte i=0; i<50; i++) {
+            assertEquals(i, data[i]);
+        }
+
+        for (int i=50; i<150; i++) {
+            assertEquals(0, data[i]);
+        }
+    }
 
     /**
      * 
@@ -278,8 +312,6 @@ public class Jdbc3BlobTest extends TestCase
         byte[] data = "abcdefghijklmnopqrstuvwxyx0123456789".getBytes();
         byte[] pattern = "def".getBytes();
 
-        String insert = "INSERT INTO " + TABLE + " VALUES (1, lo_creat(-1))";
-
         PreparedStatement ps = _conn.prepareStatement(INSERT);
         ps.executeUpdate();
         ps.close();
