@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/TypesTest.java,v 1.9 2006/05/15 09:35:57 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/TypesTest.java,v 1.12 2008/01/08 06:56:31 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -64,6 +64,20 @@ public class TypesTest extends TestCase {
         }
     }
 
+    public void testPreparedByte() throws SQLException {
+        PreparedStatement pstmt = _conn.prepareStatement("SELECT ?,?");
+        pstmt.setByte(1, (byte)1);
+        pstmt.setObject(2, new Byte((byte)2));
+        ResultSet rs = pstmt.executeQuery();
+        assertTrue(rs.next());
+        assertEquals((byte)1, rs.getByte(1));
+        assertFalse(rs.wasNull());
+        assertEquals((byte)2, rs.getByte(2));
+        assertFalse(rs.wasNull());
+        rs.close();
+        pstmt.close();
+    }
+
     public void testCallableBoolean() throws SQLException {
         CallableStatement cs = _conn.prepareCall("{? = call return_bool(?)}");
         cs.registerOutParameter(1, Types.BOOLEAN);
