@@ -1,8 +1,8 @@
 package org.postgresql.test.xa;
 
-import java.io.PrintWriter;
 import java.sql.Connection;
-import java.sql.DriverManager;
+import java.sql.Statement;
+import java.sql.ResultSet;
 
 import junit.framework.TestSuite;
 
@@ -18,7 +18,17 @@ public class XATestSuite extends TestSuite {
         {
             if (TestUtil.haveMinimumServerVersion(connection, "8.1"))
             {
-                suite.addTestSuite(XADataSourceTest.class);
+                Statement stmt = connection.createStatement();
+                ResultSet rs = stmt.executeQuery("SHOW max_prepared_transactions");
+                rs.next();
+                int mpt = rs.getInt(1);
+                if (mpt > 0) {
+                    suite.addTestSuite(XADataSourceTest.class);
+                } else {
+                    System.out.println("Skipping XA tests because max_prepared_transactions = 0.");
+                }
+                rs.close();
+                stmt.close();
             }
         }
         finally
