@@ -64,6 +64,14 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
     private static final int STATE_ACTIVE = 1;
     private static final int STATE_ENDED = 2;
 
+    /*
+     * When an XA transaction is started, we put the underlying connection
+     * into non-autocommit mode. The old setting is saved in
+     * localAutoCommitMode, so that we can restore it when the XA transaction
+     * ends and the connection returns into local transaction mode.
+     */
+    private boolean localAutoCommitMode = true;
+
     private void debug(String s) {
         logger.debug("XAResource " + Integer.toHexString(this.hashCode()) + ": " + s);
     }
@@ -152,6 +160,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
 
         try
         {
+            localAutoCommitMode = conn.getAutoCommit();
             conn.setAutoCommit(false);
         }
         catch (SQLException ex)
@@ -245,7 +254,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
             {
                 stmt.close();
             }
-            conn.setAutoCommit(true);
+            conn.setAutoCommit(localAutoCommitMode);
 
             return XA_OK;
         }
@@ -334,7 +343,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
                 state = STATE_IDLE;
                 currentXid = null;
                 conn.rollback();
-                conn.setAutoCommit(true);
+                conn.setAutoCommit(localAutoCommitMode);
             }
             else
             {
@@ -400,7 +409,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
             currentXid = null;
 
             conn.commit();
-            conn.setAutoCommit(true);
+            conn.setAutoCommit(localAutoCommitMode);
         }
         catch (SQLException ex)
         {
@@ -428,6 +437,7 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
 
             String s = RecoveredXid.xidToString(xid);
 
+            localAutoCommitMode = conn.getAutoCommit();
             conn.setAutoCommit(true);
             Statement stmt = conn.createStatement();
             try
@@ -437,11 +447,12 @@ public class PGXAConnection extends PGPooledConnection implements XAConnection,
             finally
             {
                 stmt.close();
+                conn.setAutoCommit(localAutoCommitMode);
             }
         }
         catch (SQLException ex)
         {
-            throw new XAException(ex.toString());
+            throw new PGXAException(GT.tr("Error committing prepared transaction"), ex, XAException.XAER_RMERR);
         }
     }
 
